@@ -10,10 +10,9 @@
 
 #include "config.h"
 #include "adriver.h"
+#include <linux/mm.h>
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
-#include "../alsa-kernel/core/memory_wrapper.c"
-#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 0)
 #include <sound/memalloc.h>
 #include "pci_compat_22.c"
 #endif
