@@ -50,6 +50,18 @@
 
 #include <linux/module.h>
 
+#ifdef CONFIG_HAVE_OLD_REQUEST_MODULE
+#include <linux/kmod.h>
+#undef request_module
+void snd_compat_request_module(const char *name, ...);
+#define request_module(name, args...) snd_compat_request_module(name, ##args)
+#endif
+
+#include <linux/compiler.h>
+#ifndef __user
+#define __user
+#endif
+
 #ifdef CONFIG_PCI
 #include <linux/pci.h>
 #endif
@@ -66,6 +78,7 @@
 #ifdef LINUX_2_4__donotuse
 #include <linux/init.h>
 #include <linux/pm.h>
+#include <asm/page.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 3)
 #define pci_set_dma_mask(pci, mask) pci->dma_mask = mask
 #endif
@@ -104,7 +117,8 @@
 	unsigned long name[((bits)+BITS_PER_LONG-1)/BITS_PER_LONG]
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 3)
+#include <linux/sched.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 3) && !defined(need_resched)
 #define need_resched() (current->need_resched)
 #endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 4)
@@ -122,15 +136,20 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 #endif
 
 #if defined(CONFIG_ISAPNP) || (defined(CONFIG_ISAPNP_MODULE) && defined(MODULE))
-#if (defined(CONFIG_ISAPNP_KERNEL) && defined(ALSA_BUILD)) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 3, 30) && !defined(ALSA_BUILD))
 #include <linux/isapnp.h>
+#ifndef CONFIG_PNP
+#define CONFIG_PNP
+#endif
+#if (defined(CONFIG_ISAPNP_KERNEL) && defined(ALSA_BUILD)) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 3, 30) && !defined(ALSA_BUILD))
 #define isapnp_dev pci_dev
 #define isapnp_card pci_bus
-#else
-#include <linux/isapnp.h>
 #endif
 #undef __ISAPNP__
 #define __ISAPNP__
+#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+#undef CONFIG_PNP
+#endif
 #endif
 
 #if !defined(CONFIG_ISA) && defined(CONFIG_SND_ISA)
@@ -141,11 +160,19 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 #define MODULE_LICENSE(license)
 #endif
 
-/* no vsnprintf yet? */
-/* FIXME: the version number is not sure.. at least it exists already on 2.4.10 */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 10)
-#define snprintf(buf,size,fmt,args...) sprintf(buf,fmt,##args)
-#define vsnprintf(buf,size,fmt,args) vsprintf(buf,fmt,args)
+#ifndef CONFIG_HAVE_STRLCPY
+size_t snd_compat_strlcpy(char *dest, const char *src, size_t size);
+#define strlcpy(dest, src, size) snd_compat_strlcpy(dest, src, size)
+size_t snd_compat_strlcat(char *dest, const char *src, size_t size);
+#define strlcat(dest, src, size) snd_compat_strlcat(dest, src, size)
+#endif
+
+#ifndef CONFIG_HAVE_SNPRINTF
+#include <stdarg.h>
+int snd_compat_snprintf(char * buf, size_t size, const char * fmt, ...);
+int snd_compat_vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
+#define snprintf(buf,size,fmt,args...) snd_compat_snprintf(buf,size,fmt,##args)
+#define vsnprintf(buf,size,fmt,args) snd_compat_vsnprintf(buf,size,fmt,args)
 #endif
 
 #if defined(__alpha__) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 14)
@@ -162,24 +189,16 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 28)
 #include <linux/interrupt.h>
-#include <linux/sched.h>
 static inline void synchronize_irq_wrapper(unsigned int irq) { synchronize_irq(); }
 #undef synchronize_irq
 #define synchronize_irq(irq)	synchronize_irq_wrapper(irq)
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 68)
-enum irqreturn {
-	IRQ_NONE,
-	IRQ_HANDLED
-};
-#define IRQ_RETVAL(val) (enum irqreturn)(val)
-typedef enum irqreturn irqreturn_t;
-#undef request_irq
-#define request_irq snd_compat_request_irq
-int snd_compat_request_irq(unsigned int,
-			   irqreturn_t (*handler)(int, void *, struct pt_regs *),
-			   unsigned long, const char *, void *);
-#endif /* LINUX_VERSION_CODE < 2.5.68 */
 #endif /* LINUX_VERSION_CODE < 2.5.28 */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 68)
+#define IRQ_NONE	/*void*/
+#define IRQ_HANDLED	/*void*/
+#define IRQ_RETVAL(x)	/*void*/
+typedef void irqreturn_t;
+#endif /* LINUX_VERSION_CODE < 2.5.68 */
 
 #ifndef min
 /*
@@ -202,8 +221,8 @@ int snd_compat_request_irq(unsigned int,
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 #endif
 
-#ifdef CONFIG_DEVFS_FS
 #include <linux/devfs_fs_kernel.h>
+#ifdef CONFIG_DEVFS_FS
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 29)
 #include <linux/fs.h>
 #undef register_chrdev
@@ -214,6 +233,14 @@ int snd_compat_request_irq(unsigned int,
 void snd_compat_devfs_remove(const char *fmt, ...);
 #define devfs_remove snd_compat_devfs_remove
 #endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 67)
+#undef devfs_mk_dir
+int snd_compat_devfs_mk_dir(const char *dir, ...);
+#define devfs_mk_dir snd_compat_devfs_mk_dir
+#undef devfs_mk_cdev
+int snd_compat_devfs_mk_cdev(dev_t dev, umode_t mode, const char *fmt, ...);
+#define devfs_mk_cdev snd_compat_devfs_mk_cdev
+#endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 0)
 static inline void devfs_find_and_unregister (devfs_handle_t dir, const char *name,
 					      unsigned int major, unsigned int minor,
@@ -233,10 +260,14 @@ static inline void devfs_find_and_unregister (devfs_handle_t dir, const char *na
 	devfs_unregister(master);
 }
 #endif
-#else
+#else /* !CONFIG_DEVFS_FS */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 29)
 static inline void devfs_remove(const char *fmt, ...) { }
 #endif
+#undef devfs_mk_dir
+#define devfs_mk_dir(dir, args...) do { (void)(dir); } while (0)
+#undef devfs_mk_cdev
+#define devfs_mk_cdev(dev, mode, fmt, args...) do { (void)(dev); } while (0)
 #endif /* CONFIG_DEVFS_FS */
 
 /* workarounds for USB API */
@@ -343,6 +374,16 @@ struct page *snd_compat_vmalloc_to_page(void *addr);
 #define vmalloc_to_page(addr) snd_compat_vmalloc_to_page(addr)
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 69)
+#include <linux/vmalloc.h>
+static inline void *snd_compat_vmap(struct page **pages, unsigned int count, unsigned long flags, pgprot_t prot)
+{
+	return vmap(pages, count);
+}
+#undef vmap
+#define vmap snd_compat_vmap
+#endif
+
 #include "amagic.h"
 
 #endif /* __SOUND_LOCAL_DRIVER_H */
