@@ -1,5 +1,5 @@
-#ifndef __HAL2_H
-#define __HAL2_H
+#ifndef __SOUND_HAL2_H
+#define __SOUND_HAL2_H
 
 /*
  *  Global structures used for HAL2 part of ALSA driver
@@ -18,12 +18,11 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
  */
 
 #include "pcm.h"
-#include "mixer.h"
 #include "rawmidi.h"
 #include "timer.h"
 
@@ -262,9 +261,9 @@
 
 typedef volatile unsigned long snd_hal2_reg_t;
 
-typedef struct snd_stru_hal2_ctl_regs snd_hal2_ctl_regs_t;
+typedef struct _snd_hal2_ctl_regs snd_hal2_ctl_regs_t;
 
-struct snd_stru_hal2_ctl_regs {
+struct _snd_hal2_ctl_regs {
 	snd_hal2_reg_t _unused0[4];
 	snd_hal2_reg_t isr;		/* 0x10 Status Register */
 	snd_hal2_reg_t _unused1[3];
@@ -281,9 +280,9 @@ struct snd_stru_hal2_ctl_regs {
 	snd_hal2_reg_t idr3;		/* 0x70 Indirect Data Register 3 */
 };
 
-typedef struct snd_stru_hal2_aes_regs snd_hal2_aes_regs_t;
+typedef struct _snd_hal2_aes_regs snd_hal2_aes_regs_t;
 
-struct snd_stru_hal2_aes_regs {
+struct _snd_hal2_aes_regs {
 	snd_hal2_reg_t rx_stat[2];	/* Status registers */
 	snd_hal2_reg_t rx_cr[2];	/* Control registers */
 	snd_hal2_reg_t rx_ud[4];	/* User data window */
@@ -295,16 +294,16 @@ struct snd_stru_hal2_aes_regs {
 	snd_hal2_reg_t tx_st[24];	/* Channel status data */
 };
 
-typedef struct snd_stru_hal2_vol_regs snd_hal2_vol_regs_t;
+typedef struct _snd_hal2_vol_regs snd_hal2_vol_regs_t;
 
-struct snd_stru_hal2_vol_regs {
+struct _snd_hal2_vol_regs {
 	snd_hal2_reg_t right;		/* 0x00 Right volume */
 	snd_hal2_reg_t left;		/* 0x04 Left volume */
 };
 
-typedef struct snd_stru_hal2_syn_regs snd_hal2_syn_regs_t;
+typedef struct _snd_hal2_syn_regs snd_hal2_syn_regs_t;
 
-struct snd_stru_hal2_syn_regs {
+struct _snd_hal2_syn_regs {
 	snd_hal2_reg_t _unused0[2];
 	snd_hal2_reg_t page;		/* DOC Page register */
 	snd_hal2_reg_t regsel;		/* DOC Register selection */
@@ -314,21 +313,21 @@ struct snd_stru_hal2_syn_regs {
 	snd_hal2_reg_t dram;		/* DRAM Access */
 };
 
-struct snd_stru_hal2_card;
-typedef struct snd_stru_hal2_card snd_hal2_card_t;
+struct _snd_hal2_card;
+typedef struct _snd_hal2_card snd_hal2_card_t;
 
 /* HAL2 specific structure */
 
-typedef struct snd_stru_hal2_ring snd_hal2_ring_t;
+typedef struct _snd_hal2_ring snd_hal2_ring_t;
 
-struct snd_stru_hal2_ring {
+struct _snd_hal2_ring {
 	volatile struct hpc_dma_desc desc;
 	unsigned long _padding;		/* 8 byte aligned */
 };
 
-typedef struct snd_stru_hal2_pbus snd_hal2_pbus_t;
+typedef struct _snd_hal2_pbus snd_hal2_pbus_t;
 
-struct snd_stru_hal2_pbus {
+struct _snd_hal2_pbus {
 	struct hpc3_pbus_dmacregs *pbus;
 	int pbusnr;
 	unsigned long fifobeg;
@@ -338,16 +337,16 @@ struct snd_stru_hal2_pbus {
 	unsigned long ctrl;		/* Current state of pbus->pbdma_ctrl */
 };
 
-typedef struct snd_stru_hal2_codec snd_hal2_codec_t;
+typedef struct _snd_hal2_codec snd_hal2_codec_t;
 
-struct snd_stru_hal2_codec {
+struct _snd_hal2_codec {
 	snd_dma_t *dmaptr;		/* DMA1 pointer */
 	snd_hal2_ring_t *ringbuf;	/* DMA descirptors */
 	snd_hal2_pbus_t pbus;
 	unsigned short blocks;
 };
 
-struct snd_stru_hal2 {
+struct _snd_hal2 {
 	snd_irq_t *irqptr;		/* IRQ pointer */
 	snd_hal2_ctl_regs_t *ctl_regs;	/* HAL2 ctl registers */
 	snd_hal2_aes_regs_t *aes_regs;	/* HAL2 vol registers */
@@ -364,17 +363,17 @@ struct snd_stru_hal2 {
 
 /* main structure for HAL2 card */
 
-struct snd_stru_hal2_card {
+struct _snd_hal2_card {
 	snd_card_t *card;
 	snd_pcm_t *pcm;
 	snd_rawmidi_t *midi_uart;
 
-	snd_pcm_subchn_t *playback_subchn;
-	snd_pcm_subchn_t *capture_subchn;
-	snd_pcm1_subchn_t *playback_subchn1;
-	snd_pcm1_subchn_t *capture_subchn1;
+	snd_pcm_substream_t *playback_substream;
+	snd_pcm_substream_t *capture_substream;
+	snd_pcm1_substream_t *playback_substream1;
+	snd_pcm1_substream_t *capture_substream1;
 
-	struct snd_stru_hal2 hal2;	/* HAL2 specific variables */
+	struct _snd_hal2 hal2;	/* HAL2 specific variables */
 
 	int usecount;
 };
@@ -403,4 +402,4 @@ extern void snd_hal2_interrupt(snd_hal2_card_t * hal2);
 
 extern snd_pcm_t *snd_hal2_pcm(snd_hal2_card_t *hal2);
 
-#endif				/* __HAL2_H */
+#endif /* __SOUND_HAL2_H */
